/**
 * Task Manager - Intelligent task breakdown and prioritization system
 * Analyzes loop topics and creates structured implementation roadmaps
 */

export class TaskManager {
    constructor(multiProviderAI, votingManager) {
        this.multiAI = multiProviderAI;
        this.votingManager = votingManager;
        this.activeTasks = new Map();
        this.taskHistory = [];
        this.dependencyGraph = new Map();
        
        console.log('🎯 Task Manager initialized with intelligent breakdown capabilities');
    }

    async analyzeTaskBreakdown(topic, context = {}) {
        console.log(`🔍 Analyzing task breakdown for: "${topic}"`);
        
        const analysisPrompt = `You are an expert project manager and system architect. Analyze the following topic and break it down into actionable subtasks.

TOPIC: "${topic}"
CONTEXT: ${JSON.stringify(context, null, 2)}

Provide a comprehensive task breakdown with the following structure:

MAIN_OBJECTIVE: [Clear statement of the primary goal]

SUBTASKS:
1. [Task Name] - [Brief description]
   Priority: [HIGH/MEDIUM/LOW]
   Complexity: [1-10]
   Dependencies: [List of prerequisite tasks]
   Impact: [1-10]
   Estimated Effort: [SMALL/MEDIUM/LARGE]

2. [Continue for all subtasks...]

IMPLEMENTATION_APPROACHES:
A. [Approach Name]: [Description, pros, cons]
B. [Alternative Approach]: [Description, pros, cons]

CRITICAL_PATH: [List tasks in order of execution]
PARALLEL_OPPORTUNITIES: [Tasks that can be done simultaneously]
RISK_FACTORS: [Potential issues and mitigation strategies]

Format your response exactly as shown above.`;

        try {
            const response = await this.multiAI.makeRequest(analysisPrompt, {
                maxTokens: 2000,
                temperature: 0.3
            });

            const breakdown = this.parseTaskBreakdown(response.content);
            breakdown.originalTopic = topic;
            breakdown.analysisTimestamp = new Date().toISOString();
            breakdown.id = `task-${Date.now()}`;

            this.activeTasks.set(breakdown.id, breakdown);
            this.buildDependencyGraph(breakdown);

            console.log(`✅ Task breakdown complete: ${breakdown.subtasks.length} subtasks identified`);
            return breakdown;

        } catch (error) {
            console.error('❌ Task breakdown analysis failed:', error.message);
            throw new Error(`Task analysis failed: ${error.message}`);
        }
    }

    parseTaskBreakdown(content) {
        const breakdown = {
            mainObjective: '',
            subtasks: [],
            implementationApproaches: [],
            criticalPath: [],
            parallelOpportunities: [],
            riskFactors: []
        };

        const lines = content.split('\n');
        let currentSection = '';

        for (const line of lines) {
            const trimmed = line.trim();
            
            if (trimmed.startsWith('MAIN_OBJECTIVE:')) {
                breakdown.mainObjective = trimmed.replace('MAIN_OBJECTIVE:', '').trim();
            } else if (trimmed === 'SUBTASKS:') {
                currentSection = 'subtasks';
            } else if (trimmed === 'IMPLEMENTATION_APPROACHES:') {
                currentSection = 'approaches';
            } else if (trimmed === 'CRITICAL_PATH:') {
                currentSection = 'critical';
            } else if (trimmed === 'PARALLEL_OPPORTUNITIES:') {
                currentSection = 'parallel';
            } else if (trimmed === 'RISK_FACTORS:') {
                currentSection = 'risks';
            } else if (trimmed && currentSection === 'subtasks' && /^\d+\./.test(trimmed)) {
                const task = this.parseSubtask(trimmed);
                if (task) breakdown.subtasks.push(task);
            } else if (trimmed && currentSection === 'approaches' && /^[A-Z]\./.test(trimmed)) {
                breakdown.implementationApproaches.push(trimmed);
            } else if (trimmed && currentSection === 'critical') {
                breakdown.criticalPath.push(trimmed);
            } else if (trimmed && currentSection === 'parallel') {
                breakdown.parallelOpportunities.push(trimmed);
            } else if (trimmed && currentSection === 'risks') {
                breakdown.riskFactors.push(trimmed);
            }
        }

        return breakdown;
    }

    parseSubtask(taskLine) {
        const match = taskLine.match(/^\d+\.\s*(.+?)\s*-\s*(.+)/);
        if (!match) return null;

        const [, name, description] = match;
        return {
            id: `subtask-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`,
            name: name.trim(),
            description: description.trim(),
            priority: 'MEDIUM',
            complexity: 5,
            dependencies: [],
            impact: 5,
            effort: 'MEDIUM',
            status: 'pending',
            createdAt: new Date().toISOString()
        };
    }

    buildDependencyGraph(breakdown) {
        const graph = new Map();
        
        for (const task of breakdown.subtasks) {
            graph.set(task.id, {
                task,
                dependencies: [],
                dependents: []
            });
        }

        // Analyze dependencies based on task names and descriptions
        for (const task of breakdown.subtasks) {
            const dependencies = this.identifyDependencies(task, breakdown.subtasks);
            const node = graph.get(task.id);
            node.dependencies = dependencies;
            
            // Update dependents
            for (const depId of dependencies) {
                const depNode = graph.get(depId);
                if (depNode) {
                    depNode.dependents.push(task.id);
                }
            }
        }

        this.dependencyGraph.set(breakdown.id, graph);
        return graph;
    }

    identifyDependencies(task, allTasks) {
        const dependencies = [];
        const taskWords = task.name.toLowerCase().split(' ');
        
        for (const otherTask of allTasks) {
            if (otherTask.id === task.id) continue;
            
            const otherWords = otherTask.name.toLowerCase().split(' ');
            
            // Simple dependency detection based on keywords
            if (this.hasDependencyKeywords(taskWords, otherWords)) {
                dependencies.push(otherTask.id);
            }
        }
        
        return dependencies;
    }

    hasDependencyKeywords(taskWords, otherWords) {
        const dependencyIndicators = [
            ['setup', 'configure'],
            ['design', 'implement'],
            ['create', 'test'],
            ['install', 'configure'],
            ['analyze', 'design']
        ];

        for (const [prerequisite, dependent] of dependencyIndicators) {
            if (otherWords.includes(prerequisite) && taskWords.includes(dependent)) {
                return true;
            }
        }
        
        return false;
    }

    async selectOptimalStrategy(breakdown) {
        if (breakdown.implementationApproaches.length <= 1) {
            return breakdown.implementationApproaches[0] || 'Default implementation approach';
        }

        console.log('🗳️ Using AI voting to select optimal implementation strategy');

        const votingPrompt = `Evaluate these implementation approaches for the project: "${breakdown.mainObjective}"

APPROACHES:
${breakdown.implementationApproaches.join('\n')}

CONTEXT:
- Subtasks: ${breakdown.subtasks.length}
- Critical Path Length: ${breakdown.criticalPath.length}
- Parallel Opportunities: ${breakdown.parallelOpportunities.length}
- Risk Factors: ${breakdown.riskFactors.length}

Which approach is optimal considering efficiency, maintainability, and risk mitigation?`;

        try {
            const votingResult = await this.votingManager.conductVoting(votingPrompt, {
                panel: 'reasoning',
                strategy: 'consensus',
                maxAgents: 4
            });

            console.log(`✅ Optimal strategy selected: ${votingResult.winningResponse.agentId}`);
            return votingResult.winningResponse.content;

        } catch (error) {
            console.warn('⚠️ Strategy voting failed, using first approach:', error.message);
            return breakdown.implementationApproaches[0];
        }
    }

    prioritizeTasks(breakdown) {
        const tasks = [...breakdown.subtasks];
        
        // Calculate priority scores
        tasks.forEach(task => {
            task.priorityScore = this.calculatePriorityScore(task, breakdown);
        });

        // Sort by priority score (higher is better)
        tasks.sort((a, b) => b.priorityScore - a.priorityScore);

        console.log(`📊 Tasks prioritized: ${tasks.map(t => `${t.name} (${t.priorityScore})`).join(', ')}`);
        return tasks;
    }

    calculatePriorityScore(task, breakdown) {
        const priorityWeights = { HIGH: 10, MEDIUM: 5, LOW: 1 };
        const effortWeights = { SMALL: 3, MEDIUM: 2, LARGE: 1 };
        
        const priorityScore = priorityWeights[task.priority] || 5;
        const impactScore = task.impact || 5;
        const complexityPenalty = (task.complexity || 5) * 0.5;
        const effortBonus = effortWeights[task.effort] || 2;
        
        return priorityScore + impactScore + effortBonus - complexityPenalty;
    }

    getExecutionPlan(breakdown) {
        const prioritizedTasks = this.prioritizeTasks(breakdown);
        const dependencyGraph = this.dependencyGraph.get(breakdown.id);
        
        const plan = {
            phases: [],
            parallelGroups: [],
            criticalPath: [],
            estimatedDuration: 0
        };

        // Create execution phases based on dependencies
        const processed = new Set();
        let phase = 1;

        while (processed.size < prioritizedTasks.length) {
            const currentPhase = [];
            
            for (const task of prioritizedTasks) {
                if (processed.has(task.id)) continue;
                
                const node = dependencyGraph.get(task.id);
                const dependenciesMet = node.dependencies.every(depId => processed.has(depId));
                
                if (dependenciesMet) {
                    currentPhase.push(task);
                    processed.add(task.id);
                }
            }
            
            if (currentPhase.length > 0) {
                plan.phases.push({
                    phase,
                    tasks: currentPhase,
                    canRunInParallel: currentPhase.length > 1
                });
                phase++;
            } else {
                break; // Prevent infinite loop
            }
        }

        console.log(`📋 Execution plan created: ${plan.phases.length} phases`);
        return plan;
    }

    getTaskStatus(taskId) {
        for (const [, breakdown] of this.activeTasks) {
            const task = breakdown.subtasks.find(t => t.id === taskId);
            if (task) return task;
        }
        return null;
    }

    updateTaskStatus(taskId, status, details = {}) {
        const task = this.getTaskStatus(taskId);
        if (task) {
            task.status = status;
            task.lastUpdated = new Date().toISOString();
            if (details.progress) task.progress = details.progress;
            if (details.notes) task.notes = details.notes;
            
            console.log(`📝 Task ${task.name} updated: ${status}`);
            return true;
        }
        return false;
    }

    getActiveBreakdowns() {
        return Array.from(this.activeTasks.values());
    }
}
